package co.jp.app.common;


public class ApiResponse<T> {
    //成功状况判定
    private boolean success;
    //状态码
    private int code;
    //状态信息
    private String message;
    //数据
    private T data;

    private ApiResponse(ResultCode resultCode, T data) {
        this.code = resultCode.getCode();
        this.message = resultCode.getMessage();
        this.success = (resultCode.getCode() == ResultCode.SUCCESS.getCode());
        this.data = data;
    }

    private ApiResponse() {
    }

    public static <T> ApiResponse<T> fail() {return success(null);}

    public static <T> ApiResponse<T> success(T data) {
        return new ApiResponse<>(ResultCode.SUCCESS, data);
    }

    public static <T> ApiResponse<T> fail(ResultCode resultCode) {
        if (resultCode == ResultCode.SUCCESS) {

            throw new IllegalArgumentException("Cannot use SUCCESS ResultCode for fail method. Use a specific error code.");
        }
        return new ApiResponse<>(resultCode, null);
    }

    public static <T> ApiResponse<T> fail(ResultCode resultCode, T data) {
        if (resultCode == ResultCode.SUCCESS) {
            throw new IllegalArgumentException("Cannot use SUCCESS ResultCode for fail method. Use a specific error code.");
        }
        return new ApiResponse<>(resultCode, data);
    }

    public boolean isSuccess() {
        return success;
    }

    public int getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public T getData() {
        return data;
    }

    @Override
    public String toString() {
        return "ApiResponse{" +
                "success=" + success +
                ", code=" + code +
                ", message='" + message + '\'' +
                ", data=" + data +
                '}';
    }
}
